@@ -1,5 +1,5 @@
 
-/* (C)opyleft 2001-2009 Frank DENIS <j at pureftpd dot org> */
+/* (C)opyleft 2001-2011 Frank DENIS <j at pureftpd dot org> */
 
 #include <config.h>
 
@@ -102,8 +102,8 @@ int puredbw_add(PureDBW * const dbw,
         hash0->hash1_list = newpnt;
     }
     dbw->offset_first_data += sizeof(puredb_u32_t) + sizeof(puredb_u32_t);
-    hash1 = (Hash1 *) ((unsigned char *) hash0->hash1_list +
-                       hash0->hash1_list_size - sizeof(Hash1));
+    hash1 = (Hash1 *) (void *) ((unsigned char *) hash0->hash1_list +
+                                hash0->hash1_list_size - sizeof(Hash1));
     hash1->hash = hash;
     hash1->offset_data = dbw->data_offset_counter;
     dbw->data_offset_counter += sizeof(puredb_u32_t) + sizeof(puredb_u32_t) +
@@ -261,13 +261,13 @@ static int freestructs(PureDBW * const dbw)
 
 static int mergefiles(PureDBW * const dbw)
 {
-    size_t readen;
+    size_t readnb;
     char buf[4096];
 
     rewind(dbw->fpdata);
-    while ((readen = fread(buf, (size_t) 1U, sizeof buf, dbw->fpdata)) >
+    while ((readnb = fread(buf, (size_t) 1U, sizeof buf, dbw->fpdata)) >
            (size_t) 0U) {
-        if (fwrite(buf, (size_t) 1U, readen, dbw->fpindex) != readen) {
+        if (fwrite(buf, (size_t) 1U, readnb, dbw->fpindex) != readnb) {
             return -1;
         }
     }
